/*
 * Copyright (c) 1998-2005 Jitterbit, Inc.
 *
 */
package org.jitterbit.util.xml;

import java.util.Collection;
import java.util.Comparator;

import org.apache.commons.lang.StringEscapeUtils;
import org.jitterbit.util.collections.Pair;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * Some convenience methods for handling XML.
 * 
 * @author EmanuelN
 * @author Torgil
 */
public class KongaXmlUtils {

	/**
	 * 
	 */
	private KongaXmlUtils() {
		super();
	}

    public static final String NAMESPACE = "namespace";
    
    public static final String UTF_8_PROCESSING_INSTRUCTION = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";

    /**
     * <code>Comparator</code> that sorts <code>IExpandedName</code>
     * instances by doing a case-insensitive comparison on the local name part.
     * 
     * @since 1.2
     */
    public static final Comparator<IExpandedName> SORT_EXPANDED_NAME_BY_LOCAL_PART = new Comparator<IExpandedName>() {
        
        public int compare(IExpandedName o1, IExpandedName o2) {
            return o1.getLocalName().compareToIgnoreCase(o2.getLocalName());
        }
    };
    
	/**
     * Returns the value of the attribute sName for the <tt>Node</tt> node.
     * 
     * @param node
     *            The <tt>Node</tt>
     * @param sName
     *            The value of the attribute named sName. null is returned if
     *            the value does not exist.
     */
	public static String getAttributeValue(Node node, String sName) {
		NamedNodeMap attributes = node.getAttributes();
		if ( attributes != null ) {
			Node attrNode = attributes.getNamedItem(sName);
			if ( attrNode != null ) {
				return attrNode.getNodeValue();
			}
		}
		return null;
	}
	
    /**
     * 
     * @param s
     * @throws NullPointerException
     *             if <tt>s</tt> is null.
     */
	public static String removeNamespace(String s) {
		int ipos = s.indexOf(':');
		if ( ipos > -1 ) {
			return s.substring(ipos+1);
		}
		return s;
	}
    
    /**
     * 
     * @param prefix
     * @param localName
     */
    public static String getQualifiedName(String prefix, String localName) {
        return ((prefix != null) && (prefix.length() > 0)) ? 
                        prefix + ":" + localName : localName;
    }
    
    /**
     * 
     * @param node
     * @throws NullPointerException
     *             if <tt>node</tt> is null.
     */
    public static String getLocalName(Node node) {
        String localName = node.getLocalName();
        if (localName == null) {
            String nodeName = node.getNodeName();
            localName = (nodeName == null) ? "" : removeNamespace(node.getNodeName());
        }
        return localName;
    }
    
    /**
     * 
     * @param parent
     * @throws NullPointerException
     *             if <tt>parent</tt> is null.
     */
    public static Element getFirstElementChild(Node parent) {
        Node child = parent.getFirstChild();
        while (child != null && child.getNodeType() != Node.ELEMENT_NODE) {
            child = child.getNextSibling();
        }
        return (Element) child;
    }
    
    /**
     * 
     * @param node
     * @throws NullPointerException
     *             if <tt>node</tt> is null.
     */
    public static Element getNextElementSibling(Node node) {
        Node sibling = node.getNextSibling();
        while (sibling != null && sibling.getNodeType() != Node.ELEMENT_NODE) {
            sibling = sibling.getNextSibling();
        }
        return (Element) sibling;
    }
    
    public static String getOpenElementTag(String elementName) {
        return "<" + elementName + ">";
    }
    
    public static String getEmptyElementTag(String elementName) {
        return "<" + elementName + "/>";
    }
    
    public static String getCloseElementTag(String elementName) {
        return "</" + elementName + ">";
    }
    
    public static String getElementWithValue(String elementName, String value) {
        StringBuilder sb = new StringBuilder(getOpenElementTag(elementName));
        sb.append(StringEscapeUtils.escapeXml(value));
        sb.append(getCloseElementTag(elementName));
        return sb.toString();
    }
    
    public static String getElementWithAttributes(String elementName,
                                                  Collection<Pair<String, String>> attributes,
                                                  boolean closeTag) {
        StringBuilder sb = new StringBuilder("<");
        sb.append(elementName);
        for (Pair<String, String> attr : attributes) {
            sb.append(" ").append(attr.getFirst()).append("=\"");
            sb.append(StringEscapeUtils.escapeXml(attr.getSecond())).append("\"");
        }
        if (closeTag) {
            sb.append(" />");
        }
        else {
            sb.append(">");
        }
        return sb.toString();
    }
}
